/*
 * usbdescriptor.h
 *
 *  Created on: 30.10.2012
 *      Author: Martin
 */

#ifndef USBDESCRIPTOR_H_
#define USBDESCRIPTOR_H_

#include "usbn2mc.h"

// USB device parameters
struct usb_device_descriptor PROGMEM usbserialDevice =
{
    .bLength = sizeof(struct usb_device_descriptor),
    .bDescriptorType = DEVICE,
    .bcdUSB = 0x0110,
    .bDeviceClass = 0,
    .bDeviceSubClass = 0,
    .bDeviceProtocol = 0,
    .bMaxPacketSize0 = 8,
    .idVendor = 0x1781,
    .idProduct = 0x0c62,
    .bcdDevice = 0x0300,
    .iManufacturer = 1,
    .iProduct = 2,
    .iSerialNumber = 0,
    .bNumConfigurations = 1
};

// configuration descriptor
struct
{
    struct usb_configuration_descriptor Config;
    struct usb_interface_descriptor Interface;
    struct usb_endpoint_descriptor DataInEndpoint;
    struct usb_endpoint_descriptor DataOutEndpoint;
} PROGMEM usbserialConf =
{
    .Config =
    {
        .bLength = sizeof(struct usb_configuration_descriptor),
        .bDescriptorType = CONFIGURATION,
        .wTotalLength = sizeof(usbserialConf),
        .bNumInterfaces = 1,
        .bConfigurationValue = 1,
        .iConfiguration = 0,
        .bmAttributes = 0x80,   // bus powerded
        .MaxPower = 0x1a,       // 25mA
    },
    .Interface =
    {
        .bLength = sizeof(struct usb_interface_descriptor),
        .bDescriptorType = INTERFACE,
        .bInterfaceNumber = 0,
        .bAlternateSetting = 0,
        .bNumEndpoints = 2,
        .bInterfaceClass = 0xff,
        .bInterfaceSubClass = 0,
        .bInterfaceProtocol = 0,
        .iInterface = 0,
    },
    .DataInEndpoint =
    {
        .bLength = sizeof(struct usb_endpoint_descriptor),
        .bDescriptorType = ENDPOINT,
        .bEndpointAddress = 0x81,
        .bmAttributes = 0x02, // bulk
        .wMaxPacketSize = 64,
        .bIntervall = 10,
    },
    .DataOutEndpoint =
    {
        .bLength = sizeof(struct usb_endpoint_descriptor),
        .bDescriptorType = ENDPOINT,
        .bEndpointAddress = 0x01,
        .bmAttributes = 0x02,   // bulk
        .wMaxPacketSize = 64,
        .bIntervall = 10,
    },
};

struct usb_configuration_descriptor_tab PROGMEM usbserialConfTab =
{
    .NumberOfConfigurations = 1,
    .Configurations =
    {
        (struct usb_configuration_descriptor*)&usbserialConf
    }
};

struct usb_wstring_descriptor usbserialStrLanguage PROGMEM =
{
    .bLength = 4,
    .bDescriptorType = STRING,
    .wString = {0x0409}
};

struct usb_wstring_descriptor usbserialStrManufacturer PROGMEM =
{
    .bLength = 2*10+2, //sizeof(L"krellmedia"),
    .bDescriptorType = STRING,
    .wString = {'k', 'r', 'e', 'l', 'l', 'm', 'e', 'd', 'i', 'a'} //L"krellmedia",
};

struct usb_wstring_descriptor PROGMEM usbserialStrProduct =
{
    .bLength = 2*8+2, //sizeof(L"USBtoSPI"),
    .bDescriptorType = STRING,
    .wString = {'U', 'S', 'B', 't', 'o', 'S', 'P', 'I'} //L"USBtoSPI"
};

struct usb_wstring_descriptor_tab PROGMEM StringTab =
{
    .NumberOfStrings = 3,
    .Strings =
    {
        (struct usb_wstring_descriptor*)&usbserialStrLanguage,
        (struct usb_wstring_descriptor*)&usbserialStrManufacturer,
        (struct usb_wstring_descriptor*)&usbserialStrProduct
    }
};

#endif /* USBDESCRIPTOR_H_ */
